@@ -15,7 +15,7 @@ class PluginFunctionHtmlSelectDateTests extends PHPUnit_Framework_TestCase {
         $this->smarty = SmartyTests::$smarty;
         SmartyTests::init();
         
-        $this->now = mktime( 15, 0, 0, 2, 20, 2011 );
+        $this->now = mktime( 15, 0, 0, 2, 20, 2013 );
     } 
     
     public static function isRunnable()
@@ -32,42 +32,59 @@ class PluginFunctionHtmlSelectDateTests extends PHPUnit_Framework_TestCase {
 <option value="2008">2008</option>
 <option value="2009">2009</option>
 <option value="2010">2010</option>
-<option value="2011" selected="selected">2011</option>',
-        'start_+5' => '<option value="2011" selected="selected">2011</option>
+<option value="2011">2011</option>
 <option value="2012">2012</option>
-<option value="2013">2013</option>
+<option value="2013" selected="selected">2013</option>',
+        'start_+5' => '<option value="2013" selected="selected">2013</option>
 <option value="2014">2014</option>
 <option value="2015">2015</option>
-<option value="2016">2016</option>',
-        'start_-5' => '<option value="2006">2006</option>
-<option value="2007">2007</option>
-<option value="2008">2008</option>
+<option value="2016">2016</option>
+<option value="2017">2017</option>
+<option value="2018">2018</option>',
+        'start_-5' => '<option value="2008">2008</option>
 <option value="2009">2009</option>
 <option value="2010">2010</option>
-<option value="2011" selected="selected">2011</option>',
+<option value="2011">2011</option>
+<option value="2012">2012</option>
+<option value="2013" selected="selected">2013</option>',
         'end_2005' => '<option value="2005">2005</option>
 <option value="2006">2006</option>
 <option value="2007">2007</option>
 <option value="2008">2008</option>
 <option value="2009">2009</option>
 <option value="2010">2010</option>
-<option value="2011" selected="selected">2011</option>',
-        'end_+5' => '<option value="2011" selected="selected">2011</option>
+<option value="2011">2011</option>
 <option value="2012">2012</option>
-<option value="2013">2013</option>
+<option value="2013" selected="selected">2013</option>',
+        'end_+5' => '<option value="2013" selected="selected">2013</option>
 <option value="2014">2014</option>
 <option value="2015">2015</option>
-<option value="2016">2016</option>',
-        'end_-5' => '<option value="2006">2006</option>
-<option value="2007">2007</option>
-<option value="2008">2008</option>
+<option value="2016">2016</option>
+<option value="2017">2017</option>
+<option value="2018">2018</option>',
+        'end_-5' => '<option value="2008">2008</option>
 <option value="2009">2009</option>
 <option value="2010">2010</option>
-<option value="2011" selected="selected">2011</option>',
-        'default' => '<option value="2011" selected="selected">2011</option>',
+<option value="2011">2011</option>
+<option value="2012">2012</option>
+<option value="2013" selected="selected">2013</option>',
+        'default' => '<option value="2013" selected="selected">2013</option>',
+        'none' => '<option value="2013">2013</option>',
     );
     
     protected $months = array(
+        'none' => '<option value="01">January</option>
+<option value="02">February</option>
+<option value="03">March</option>
+<option value="04">April</option>
+<option value="05">May</option>
+<option value="06">June</option>
+<option value="07">July</option>
+<option value="08">August</option>
+<option value="09">September</option>
+<option value="10">October</option>
+<option value="11">November</option>
+<option value="12">December</option>',
         'default' => '<option value="01">January</option>
 <option value="02" selected="selected">February</option>
 <option value="03">March</option>
@@ -119,6 +136,37 @@ class PluginFunctionHtmlSelectDateTests extends PHPUnit_Framework_TestCase {
     );
     
     protected $days = array(
+        'none' => '<option value="1">01</option>
+<option value="2">02</option>
+<option value="3">03</option>
+<option value="4">04</option>
+<option value="5">05</option>
+<option value="6">06</option>
+<option value="7">07</option>
+<option value="8">08</option>
+<option value="9">09</option>
+<option value="10">10</option>
+<option value="11">11</option>
+<option value="12">12</option>
+<option value="13">13</option>
+<option value="14">14</option>
+<option value="15">15</option>
+<option value="16">16</option>
+<option value="17">17</option>
+<option value="18">18</option>
+<option value="19">19</option>
+<option value="20">20</option>
+<option value="21">21</option>
+<option value="22">22</option>
+<option value="23">23</option>
+<option value="24">24</option>
+<option value="25">25</option>
+<option value="26">26</option>
+<option value="27">27</option>
+<option value="28">28</option>
+<option value="29">29</option>
+<option value="30">30</option>
+<option value="31">31</option>',
         'default' => '<option value="1">01</option>
 <option value="2">02</option>
 <option value="3">03</option>
@@ -342,6 +390,34 @@ class PluginFunctionHtmlSelectDateTests extends PHPUnit_Framework_TestCase {
         $this->assertEquals($result, $this->smarty->fetch($tpl));
     }
     
+    public function testEmptyUnset()
+    {
+        $n = "\n";
+        $result = '<select name="Date_Month">'.$n.'<option value=""></option>'.$n. $this->months['none'] .$n.'</select>'
+            .$n.'<select name="Date_Day">'.$n.'<option value=""></option>'.$n. $this->days['none'] .$n.'</select>'
+            .$n.'<select name="Date_Year">'.$n.'<option value=""></option>'.$n. $this->years['none'] .$n.'</select>';
+        $tpl = $this->smarty->createTemplate('eval:{html_select_date time=null all_empty=""}');
+        $this->assertEquals($result, $this->smarty->fetch($tpl));
+        
+        $result = '<select name="Date_Month">'.$n.'<option value="">all</option>'.$n. $this->months['none'] .$n.'</select>'
+            .$n.'<select name="Date_Day">'.$n.'<option value="">all</option>'.$n. $this->days['none'] .$n.'</select>'
+            .$n.'<select name="Date_Year">'.$n.'<option value="">all</option>'.$n. $this->years['none'] .$n.'</select>';
+        $tpl = $this->smarty->createTemplate('eval:{html_select_date time=null all_empty="all"}');
+        $this->assertEquals($result, $this->smarty->fetch($tpl));
+        
+        $result = '<select name="Date_Month">'.$n. $this->months['none'] .$n.'</select>'
+            .$n.'<select name="Date_Day">'.$n. $this->days['none'] .$n.'</select>'
+            .$n.'<select name="Date_Year">'.$n.'<option value=""></option>'.$n. $this->years['none'] .$n.'</select>';
+        $tpl = $this->smarty->createTemplate('eval:{html_select_date time=null year_empty=""}');
+        $this->assertEquals($result, $this->smarty->fetch($tpl));
+        
+        $result = '<select name="Date_Month">'.$n.'<option value="">month</option>'.$n. $this->months['none'] .$n.'</select>'
+            .$n.'<select name="Date_Day">'.$n.'<option value="">day</option>'.$n. $this->days['none'] .$n.'</select>'
+            .$n.'<select name="Date_Year">'.$n.'<option value="">year</option>'.$n. $this->years['none'] .$n.'</select>';
+        $tpl = $this->smarty->createTemplate('eval:{html_select_date time=null year_empty="year" month_empty="month" day_empty="day"}');
+        $this->assertEquals($result, $this->smarty->fetch($tpl));
+    }
+    
     public function testId()
     {
         $n = "\n";
@@ -468,13 +544,13 @@ class PluginFunctionHtmlSelectDateTests extends PHPUnit_Framework_TestCase {
         $n = "\n";
         $result = '<select name="Date_Month">'.$n. $this->months['default'] .$n.'</select>'
             .$n.'<select name="Date_Day">'.$n. $this->days['default'] .$n.'</select>'
-            .$n.'<input type="text" name="Date_Year" value="2011" size="4" maxlength="4" />';
+            .$n.'<input type="text" name="Date_Year" value="2013" size="4" maxlength="4" />';
         $tpl = $this->smarty->createTemplate('eval:{html_select_date time='. $this->now .' year_as_text=true}');
         $this->assertEquals($result, $this->smarty->fetch($tpl));
         
         $result = '<select name="foo_Month">'.$n. $this->months['default'] .$n.'</select>'
             .$n.'<select name="foo_Day">'.$n. $this->days['default'] .$n.'</select>'
-            .$n.'<input type="text" name="foo_Year" value="2011" size="4" maxlength="4" />';
+            .$n.'<input type="text" name="foo_Year" value="2013" size="4" maxlength="4" />';
         $tpl = $this->smarty->createTemplate('eval:{html_select_date time='. $this->now .' year_as_text=true prefix="foo_"}');
         $this->assertEquals($result, $this->smarty->fetch($tpl));
     }
@@ -542,7 +618,7 @@ class PluginFunctionHtmlSelectDateTests extends PHPUnit_Framework_TestCase {
             'namorized' => array(
                 'foobar_Month' => '02',
                 'foobar_Day' => '20',
-                'foobar_Year' => '2011',
+                'foobar_Year' => '2013',
             ),
         );
 
