@@ -342,7 +342,7 @@ class PHP_LexerGenerator_Parser#line 171 "Parser.php"
         $pattern .= '/' . $this->patternFlags;
         fwrite($this->out, '
         $tokenMap = ' . $tokenindex . ';
-        if (' . $this->counter . ' >= strlen(' . $this->input . ')) {
+        if (' . $this->counter . ' >= ($this->mbstring_overload ? mb_strlen(' . $this->input . ',\'latin1\'): strlen(' . $this->input . '))) {
             return false; // end of input
         }
         ');
@@ -350,7 +350,9 @@ class PHP_LexerGenerator_Parser#line 171 "Parser.php"
             $pattern . 'iS";' . "\n");
         fwrite($this->out, '
         do {
-            if (preg_match($yy_global_pattern,' . $this->input . ', $yymatches, null, ' .
+            if ($this->mbstring_overload ? preg_match($yy_global_pattern, mb_substr(' . $this->input . ', ' .
+             $this->counter .
+                    ',2000000000,\'latin1\'), $yymatches) : preg_match($yy_global_pattern,' . $this->input . ', $yymatches, null, ' .
              $this->counter .
                     ')) {
                 $yysubmatches = $yymatches;
@@ -372,7 +374,7 @@ class PHP_LexerGenerator_Parser#line 171 "Parser.php"
                 ' . $this->value . ' = current($yymatches); // token value
                 $r = $this->{\'yy_r' . $ruleindex . '_\' . ' . $this->token . '}($yysubmatches);
                 if ($r === null) {
-                    ' . $this->counter . ' += strlen(' . $this->value . ');
+                    ' . $this->counter . ' += ($this->mbstring_overload ? mb_strlen(' . $this->value . ',\'latin1\'): strlen(' . $this->value . '));
                     ' . $this->line . ' += substr_count(' . $this->value . ', "\n");
                     // accept this token
                     return true;
@@ -381,9 +383,9 @@ class PHP_LexerGenerator_Parser#line 171 "Parser.php"
                     // process this token in the new state
                     return $this->yylex();
                 } elseif ($r === false) {
-                    ' . $this->counter . ' += strlen(' . $this->value . ');
+                    ' . $this->counter . ' += ($this->mbstring_overload ? mb_strlen(' . $this->value . ',\'latin1\'): strlen(' . $this->value . '));
                     ' . $this->line . ' += substr_count(' . $this->value . ', "\n");
-                    if (' . $this->counter . ' >= strlen(' . $this->input . ')) {
+                    if (' . $this->counter . ' >= ($this->mbstring_overload ? mb_strlen(' . $this->input . ',\'latin1\'): strlen(' . $this->input . '))) {
                         return false; // end of input
                     }
                     // skip this token
@@ -430,16 +432,16 @@ class PHP_LexerGenerator_Parser#line 171 "Parser.php"
                         // process this token in the new state
                         return $this->yylex();
                     } elseif ($r === false) {
-                        ' . $this->counter . ' += strlen(' . $this->value . ');
+                        ' . $this->counter . ' += ($this->mbstring_overload ? mb_strlen(' . $this->value . ',\'latin1\'): strlen(' . $this->value . '));
                         ' . $this->line . ' += substr_count(' . $this->value . ', "\n");
-                        if (' . $this->counter . ' >= strlen(' . $this->input . ')) {
+                        if (' . $this->counter . ' >= ($this->mbstring_overload ? mb_strlen(' . $this->input . ',\'latin1\'): strlen(' . $this->input . '))) {
                             return false; // end of input
                         }
                         // skip this token
                         continue;
                     } else {
                         // accept
-                        ' . $this->counter . ' += strlen(' . $this->value . ');
+                        ' . $this->counter . ' += ($this->mbstring_overload ? mb_strlen(' . $this->value . ',\'latin1\'): strlen(' . $this->value . '));
                         ' . $this->line . ' += substr_count(' . $this->value . ', "\n");
                         return true;
                     }
@@ -527,10 +529,10 @@ class PHP_LexerGenerator_Parser#line 171 "Parser.php"
 
 /* Next is all token values, as class constants
 */
-/* 
+/*
 ** These constants (all generated automatically by the parser generator)
 ** specify the various kinds of tokens (terminals) that the parser
-** understands. 
+** understands.
 **
 ** Each symbol here is a terminal symbol in the grammar.
 */
@@ -550,7 +552,7 @@ class PHP_LexerGenerator_Parser#line 171 "Parser.php"
 /* Next are that tables used to determine what action to take based on the
 ** current state and lookahead token.  These tables are used to implement
 ** functions that take a state number and lookahead value and return an
-** action integer.  
+** action integer.
 **
 ** Suppose the action integer is N.  Then the action is determined as
 ** follows
@@ -578,7 +580,7 @@ class PHP_LexerGenerator_Parser#line 171 "Parser.php"
 ** If the index value self::$yy_shift_ofst[S]+X is out of range or if the value
 ** self::$yy_lookahead[self::$yy_shift_ofst[S]+X] is not equal to X or if
 ** self::$yy_shift_ofst[S] is equal to self::YY_SHIFT_USE_DFLT, it means that
-** the action is not in the table and that self::$yy_default[S] should be used instead.  
+** the action is not in the table and that self::$yy_default[S] should be used instead.
 **
 ** The formula above is for computing the action when the lookahead is
 ** a terminal symbol.  If the lookahead is a non-terminal (as occurs after
@@ -712,7 +714,7 @@ static public $yy_action = array(
 ** various aspects of the generated parser.
 **    self::YYNOCODE      is a number which corresponds
 **                        to no legal terminal or nonterminal number.  This
-**                        number is used to fill in empty slots of the hash 
+**                        number is used to fill in empty slots of the hash
 **                        table.
 **    self::YYFALLBACK    If defined, this indicates that one or more tokens
 **                        have fall-back values which should be used if the
@@ -732,7 +734,7 @@ static public $yy_action = array(
     const YYFALLBACK = 0;
     /** The next table maps tokens into fallback tokens.  If a construct
      * like the following:
-     * 
+     *
      *      %fallback ID X Y Z.
      *
      * appears in the grammer, then ID becomes a fallback token for X, Y,
@@ -745,10 +747,10 @@ static public $yy_action = array(
     /**
      * Turn parser tracing on by giving a stream to which to write the trace
      * and a prompt to preface each trace message.  Tracing is turned off
-     * by making either argument NULL 
+     * by making either argument NULL
      *
      * Inputs:
-     * 
+     *
      * - A stream resource to which trace output should be written.
      *   If NULL, then tracing is turned off.
      * - A prefix string written at the beginning of every
@@ -756,7 +758,7 @@ static public $yy_action = array(
      *   turned off.
      *
      * Outputs:
-     * 
+     *
      * - None.
      * @param resource
      * @param string
@@ -808,12 +810,12 @@ static public $yy_action = array(
      * are required.  The following table supplies these names
      * @var array
      */
-    static public $yyTokenName = array( 
-  '$',             'PHPCODE',       'COMMENTSTART',  'COMMENTEND',  
-  'PI',            'SUBPATTERN',    'CODE',          'PATTERN',     
-  'QUOTE',         'SINGLEQUOTE',   'error',         'start',       
+    static public $yyTokenName = array(
+  '$',             'PHPCODE',       'COMMENTSTART',  'COMMENTEND',
+  'PI',            'SUBPATTERN',    'CODE',          'PATTERN',
+  'QUOTE',         'SINGLEQUOTE',   'error',         'start',
   'lexfile',       'declare',       'rules',         'declarations',
-  'processing_instructions',  'pattern_declarations',  'subpattern',    'rule',        
+  'processing_instructions',  'pattern_declarations',  'subpattern',    'rule',
   'reset_rules',   'rule_subpattern',
     );
 
@@ -890,7 +892,7 @@ static public $yy_action = array(
         /* Here is inserted the actions which take place when a
         ** terminal or non-terminal is destroyed.  This can happen
         ** when the symbol is popped from the stack during a
-        ** reduce or during error processing or when a parser is 
+        ** reduce or during error processing or when a parser is
         ** being destroyed before it is finished parsing.
         **
         ** Note: during a reduce, the only symbols destroyed are those
@@ -1017,7 +1019,7 @@ static public $yy_action = array(
     /**
      * Based on the parser state and current parser stack, determine whether
      * the lookahead token is possible.
-     * 
+     *
      * The parser will convert the token value to an error token if not.  This
      * catches some unusual edge cases where the parser would fail.
      * @param int
@@ -1106,7 +1108,7 @@ static public $yy_action = array(
     function yy_find_shift_action($iLookAhead)
     {
         $stateno = $this->yystack[$this->yyidx]->stateno;
-     
+
         /* if ($this->yyidx < 0) return self::YY_NO_ACTION;  */
         if (!isset(self::$yy_shift_ofst[$stateno])) {
             // no shift actions
@@ -1263,7 +1265,7 @@ static public $yy_action = array(
     /**
      * The following table contains a mapping of reduce action to method name
      * that handles the reduction.
-     * 
+     *
      * If a rule is not set, it has no handler.
      */
     static public $yyReduceMap = array(
@@ -1720,14 +1722,14 @@ static public $yy_action = array(
 
     /**
      * placeholder for the left hand side in a reduce operation.
-     * 
+     *
      * For a parser with a rule like this:
      * <pre>
      * rule(A) ::= B. { A = 1; }
      * </pre>
-     * 
+     *
      * The parser will translate to something like:
-     * 
+     *
      * <code>
      * function yy_r0(){$this->_retvalue = 1;}
      * </code>
@@ -1737,13 +1739,13 @@ static public $yy_action = array(
     /**
      * Perform a reduce action and the shift that must immediately
      * follow the reduce.
-     * 
+     *
      * For a rule such as:
-     * 
+     *
      * <pre>
      * A ::= B blah C. { dosomething(); }
      * </pre>
-     * 
+     *
      * This function will first call the action, if any, ("dosomething();" in our
      * example), and then it will pop three states from the stack,
      * one for each entry on the right-hand side of the expression
@@ -1760,7 +1762,7 @@ static public $yy_action = array(
         //PHP_LexerGenerator_ParseryyStackEntry $yymsp;            /* The top of the parser's stack */
         //int $yysize;                     /* Amount to pop the stack */
         $yymsp = $this->yystack[$this->yyidx];
-        if (self::$yyTraceFILE && $yyruleno >= 0 
+        if (self::$yyTraceFILE && $yyruleno >= 0
               && $yyruleno < count(self::$yyRuleName)) {
             fprintf(self::$yyTraceFILE, "%sReduce (%d) [%s].\n",
                 self::$yyTracePrompt, $yyruleno,
@@ -1804,7 +1806,7 @@ static public $yy_action = array(
 
     /**
      * The following code executes when the parse fails
-     * 
+     *
      * Code from %parse_fail is inserted here
      */
     function yy_parse_failed()
@@ -1821,7 +1823,7 @@ static public $yy_action = array(
 
     /**
      * The following code executes when a syntax error first occurs.
-     * 
+     *
      * %syntax_error code is inserted here
      * @param int The major type of the error token
      * @param mixed The minor type of the error token
@@ -1845,7 +1847,7 @@ static public $yy_action = array(
 
     /**
      * The following is executed when the parser accepts
-     * 
+     *
      * %parse_accept code is inserted here
      */
     function yy_accept()
@@ -1862,7 +1864,7 @@ static public $yy_action = array(
 
     /**
      * The main parser program.
-     * 
+     *
      * The first argument is the major token number.  The second is
      * the token value string as scanned from the input.
      *
@@ -1875,7 +1877,7 @@ static public $yy_action = array(
 //        $yyact;            /* The parser action. */
 //        $yyendofinput;     /* True if we are at the end of input */
         $yyerrorhit = 0;   /* True if yymajor has invoked an error */
-        
+
         /* (re)initialize the parser, if necessary */
         if ($this->yyidx === null || $this->yyidx < 0) {
             /* if ($yymajor == 0) return; // not sure why this was here... */
@@ -1888,12 +1890,12 @@ static public $yy_action = array(
             array_push($this->yystack, $x);
         }
         $yyendofinput = ($yymajor==0);
-        
+
         if (self::$yyTraceFILE) {
             fprintf(self::$yyTraceFILE, "%sInput %s\n",
                 self::$yyTracePrompt, self::$yyTokenName[$yymajor]);
         }
-        
+
         do {
             $yyact = $this->yy_find_shift_action($yymajor);
             if ($yymajor < self::YYERRORSYMBOL &&
@@ -1919,7 +1921,7 @@ static public $yy_action = array(
                 if (self::YYERRORSYMBOL) {
                     /* A syntax error has occurred.
                     ** The response to an error depends upon whether or not the
-                    ** grammar defines an error token "ERROR".  
+                    ** grammar defines an error token "ERROR".
                     **
                     ** This is what we do if the grammar does define ERROR:
                     **
@@ -1989,7 +1991,7 @@ static public $yy_action = array(
             } else {
                 $this->yy_accept();
                 $yymajor = self::YYNOCODE;
-            }            
+            }
         } while ($yymajor != self::YYNOCODE && $this->yyidx >= 0);
     }
 }
\ No newline at end of file
