@@ -24,8 +24,10 @@ function smarty_modifiercompiler_default ($params, $compiler)
     if (!isset($params[1])) {
         $params[1] = "''";
     }
-    for ($i = 1, $cnt = count($params); $i < $cnt; $i++) {
-        $output = '(($tmp = @' . $output . ')===null||$tmp===\'\' ? ' . $params[$i] . ' : $tmp)';
+    
+    array_shift($params);
+    foreach ($params as $param) {
+        $output = '(($tmp = @' . $output . ')===null||$tmp===\'\' ? ' . $param . ' : $tmp)';
     }
     return $output;
 }
