@@ -30,7 +30,7 @@ abstract class Smarty_Resource {
      * cache for Smarty_Resource instances
      * @var array
      */
-    protected static $resources = array();
+    public static $resources = array();
     /**
      * resource types provided by the core
      * @var array
@@ -91,6 +91,19 @@ abstract class Smarty_Resource {
         // intentionally left blank
     }
 
+
+    /**
+     * modify resource_name according to resource handlers specifications
+     *
+     * @param Smarty $smarty        Smarty instance
+     * @param string $resource_name resource_name to make unique
+     * @return string unique resource name
+     */
+    protected function buildUniqueResourceName(Smarty $smarty, $resource_name)
+    {
+        return get_class($this) . '#' . $smarty->joined_template_dir . '#' . $resource_name;
+    }
+
     /**
      * populate Compiled Object with compiled filepath
      *
@@ -131,7 +144,52 @@ abstract class Smarty_Resource {
 
         $compiled->filepath = $_compile_dir . $_filepath . '.' . $compiled->source->type . $_basename . $_cache . '.php';
     }
-
+    
+    /**
+     * Normalize Paths "foo/../bar" to "bar" 
+     *
+     * @param string $_path path to normalize
+     * @param boolean $ds respect windows directory separator
+     * @return string normalized path
+     */
+    protected function normalizePath($_path, $ds=true)
+    {
+        if ($ds) {
+            // don't we all just love windows?
+            $_path = str_replace('\\', '/', $_path);
+        }
+        
+        $offset = 0;
+        
+        // resolve simples
+        $_path = preg_replace('#(/\./(\./)*)|/{2,}#', '/', $_path);
+        // resolve parents
+        while (true) {
+            $_parent = strpos($_path, '/../', $offset);
+            if (!$_parent) {
+                break;
+            } else if ($_path[$_parent - 1] === '.') {
+                $offset = $_parent + 3;
+                continue;
+            }
+            
+            $_pos = strrpos($_path, '/', $_parent - strlen($_path) - 1);
+            if ($_pos === false) {
+                // don't we all just love windows?
+                $_pos = $_parent;
+            }
+            
+            $_path = substr_replace($_path, '', $_pos, $_parent + 3 - $_pos);
+        }
+        
+        if ($ds && DS != '/') {
+            // don't we all just love windows?
+            $_path = str_replace('/', '\\', $_path);
+        }
+        
+        return $_path;
+    }
+    
     /**
      * build template filepath by traversing the template_dir array
      *
@@ -154,42 +212,29 @@ abstract class Smarty_Resource {
         // go relative to a given template?
         $_file_is_dotted = $file[0] == '.' && ($file[1] == '.' || $file[1] == '/' || $file[1] == "\\");
         if ($_template && $_template->parent instanceof Smarty_Internal_Template && $_file_is_dotted) {
-            if ($_template->parent->source->type != 'file' && $_template->parent->source->type != 'extends') {
+            if ($_template->parent->source->type != 'file' && $_template->parent->source->type != 'extends' && !$_template->parent->allow_relative_path) {
                 throw new SmartyException("Template '{$file}' cannot be relative to template of resource type '{$_template->parent->source->type}'");
             }
             $file = dirname($_template->parent->source->filepath) . DS . $file;
             $_file_exact_match = true;
-        } elseif ($_file_is_dotted) {
-            throw new SmartyException("Template '{$file}' may not start with ../ or ./'");
+            if (!preg_match('/^([\/\\\\]|[a-zA-Z]:[\/\\\\])/', $file)) {
+                // the path gained from the parent template is relative to the current working directory
+                // as expansions (like include_path) have already been done
+                $file = getcwd() . DS . $file;
+            }
         }
 
         // resolve relative path
         if (!preg_match('/^([\/\\\\]|[a-zA-Z]:[\/\\\\])/', $file)) {
-            $_path = DS . trim($file, '/\\');
+            // don't we all just love windows?
+            $_path = str_replace('\\', '/', $file);
+            $_path = DS . trim($file, '/');
             $_was_relative = true;
         } else {
-            $_path = $file;
-        }
-        // don't we all just love windows?
-        $_path = str_replace('\\', '/', $_path);
-        // resolve simples
-        $_path = preg_replace('#(/\./(\./)*)|/{2,}#', '/', $_path);
-        // resolve parents
-        while (true) {
-            $_parent = strpos($_path, '/../');
-            if ($_parent === false) {
-                break;
-            } else if ($_parent === 0) {
-                $_path = substr($_path, 3);
-                break;
-            }
-            $_pos = strrpos($_path, '/', $_parent - strlen($_path) - 1);
-            if ($_pos === false) {
-                // don't we all just love windows?
-                $_pos = $_parent;
-            }
-            $_path = substr_replace($_path, '', $_pos, $_parent + 3 - $_pos);
+            // don't we all just love windows?
+            $_path = str_replace('\\', '/', $file);
         }
+        $_path = $this->normalizePath($_path, false);
         if (DS != '/') {
             // don't we all just love windows?
             $_path = str_replace('/', '\\', $_path);
@@ -198,6 +243,7 @@ abstract class Smarty_Resource {
         if (isset($_was_relative)) {
             $_path = substr($_path, 1);
         }
+
         // this is only required for directories
         $file = rtrim($_path, '/\\');
 
@@ -208,10 +254,6 @@ abstract class Smarty_Resource {
 
         // template_dir index?
         if (preg_match('#^\[(?P<key>[^\]]+)\](?P<file>.+)$#', $file, $match)) {
-            if ($match['file'][0] == '.' && ($match['file'][1] == '.' || $match['file'][1] == '/' || $match['file'][1] == "\\")) {
-                throw new SmartyException("Template '{$match['file']}' may not start with ../ or ./'");
-            }
-
             $_directory = null;
             // try string indexes
             if (isset($_directories[$match['key']])) {
@@ -237,17 +279,27 @@ abstract class Smarty_Resource {
             }
         }
 
+        $_stream_resolve_include_path = function_exists('stream_resolve_include_path');
+
         // relative file name?
         if (!preg_match('/^([\/\\\\]|[a-zA-Z]:[\/\\\\])/', $file)) {
             foreach ($_directories as $_directory) {
                 $_filepath = $_directory . $file;
                 if ($this->fileExists($source, $_filepath)) {
-                    return $_filepath;
+                    return $this->normalizePath($_filepath);
                 }
                 if ($source->smarty->use_include_path && !preg_match('/^([\/\\\\]|[a-zA-Z]:[\/\\\\])/', $_directory)) {
                     // try PHP include_path
-                    if (($_filepath = Smarty_Internal_Get_Include_Path::getIncludePath($_filepath)) !== false) {
-                        return $_filepath;
+                    if ($_stream_resolve_include_path) {
+                        $_filepath = stream_resolve_include_path($_filepath);
+                    } else {
+                        $_filepath = Smarty_Internal_Get_Include_Path::getIncludePath($_filepath);
+                    }
+
+                    if ($_filepath !== false) {
+                        if ($this->fileExists($source, $_filepath)) {
+                            return $this->normalizePath($_filepath);
+                        }
                     }
                 }
             }
@@ -313,67 +365,132 @@ abstract class Smarty_Resource {
     /**
      * Load Resource Handler
      *
-     * @param Smarty $smarty        smarty object
-     * @param string $resource_type name of the resource
+     * @param Smarty $smarty    smarty object
+     * @param string $type      name of the resource
      * @return Smarty_Resource Resource Handler
      */
-    public static function load(Smarty $smarty, $resource_type)
+    public static function load(Smarty $smarty, $type)
     {
-        // try the instance cache
-        if (isset(self::$resources[$resource_type])) {
-            return self::$resources[$resource_type];
+        // try smarty's cache
+        if (isset($smarty->_resource_handlers[$type])) {
+            return $smarty->_resource_handlers[$type];
         }
 
         // try registered resource
-        if (isset($smarty->registered_resources[$resource_type])) {
-            if ($smarty->registered_resources[$resource_type] instanceof Smarty_Resource) {
-                return self::$resources[$resource_type] = $smarty->registered_resources[$resource_type];
+        if (isset($smarty->registered_resources[$type])) {
+            if ($smarty->registered_resources[$type] instanceof Smarty_Resource) {
+                $smarty->_resource_handlers[$type] = $smarty->registered_resources[$type];
+                // note registered to smarty is not kept unique!
+                return $smarty->_resource_handlers[$type];
             }
+
             if (!isset(self::$resources['registered'])) {
                 self::$resources['registered'] = new Smarty_Internal_Resource_Registered();
             }
-            return self::$resources['registered'];
+            if (!isset($smarty->_resource_handlers[$type])) {
+                $smarty->_resource_handlers[$type] = self::$resources['registered'];
+            }
+
+            return $smarty->_resource_handlers[$type];
         }
 
         // try sysplugins dir
-        if (isset(self::$sysplugins[$resource_type])) {
-            $_resource_class = 'Smarty_Internal_Resource_' . ucfirst($resource_type);
-            return self::$resources[$resource_type] = new $_resource_class();
+        if (isset(self::$sysplugins[$type])) {
+            if (!isset(self::$resources[$type])) {
+                $_resource_class = 'Smarty_Internal_Resource_' . ucfirst($type);
+                self::$resources[$type] = new $_resource_class();
+            }
+            return $smarty->_resource_handlers[$type] = self::$resources[$type];
         }
 
         // try plugins dir
-        $_resource_class = 'Smarty_Resource_' . ucfirst($resource_type);
+        $_resource_class = 'Smarty_Resource_' . ucfirst($type);
         if ($smarty->loadPlugin($_resource_class)) {
+            if (isset(self::$resources[$type])) {
+                return $smarty->_resource_handlers[$type] = self::$resources[$type];
+            }
+
             if (class_exists($_resource_class, false)) {
-                return self::$resources[$resource_type] = new $_resource_class();
+                self::$resources[$type] = new $_resource_class();
+                return $smarty->_resource_handlers[$type] = self::$resources[$type];
             } else {
-                $smarty->registerResource($resource_type,
-                    array("smarty_resource_{$resource_type}_source",
-                        "smarty_resource_{$resource_type}_timestamp",
-                        "smarty_resource_{$resource_type}_secure",
-                        "smarty_resource_{$resource_type}_trusted"));
+                $smarty->registerResource($type, array(
+                    "smarty_resource_{$type}_source",
+                    "smarty_resource_{$type}_timestamp",
+                    "smarty_resource_{$type}_secure",
+                    "smarty_resource_{$type}_trusted"
+                ));
+
                 // give it another try, now that the resource is registered properly
-                return self::load($smarty, $resource_type);
+                return self::load($smarty, $type);
             }
         }
 
         // try streams
         $_known_stream = stream_get_wrappers();
-        if (in_array($resource_type, $_known_stream)) {
+        if (in_array($type, $_known_stream)) {
             // is known stream
             if (is_object($smarty->security_policy)) {
-                $smarty->security_policy->isTrustedStream($resource_type);
+                $smarty->security_policy->isTrustedStream($type);
             }
             if (!isset(self::$resources['stream'])) {
                 self::$resources['stream'] = new Smarty_Internal_Resource_Stream();
             }
-            return self::$resources['stream'];
+            return $smarty->_resource_handlers[$type] = self::$resources['stream'];
         }
 
         // TODO: try default_(template|config)_handler
 
         // give up
-        throw new SmartyException('Unkown resource type \'' . $resource_type . '\'');
+        throw new SmartyException("Unkown resource type '{$type}'");
+    }
+
+    /**
+     * extract resource_type and resource_name from template_resource and config_resource
+     *
+     * @note "C:/foo.tpl" was forced to file resource up till Smarty 3.1.3 (including).
+     * @param string  $resource_name    template_resource or config_resource to parse
+     * @param string  $default_resource the default resource_type defined in $smarty
+     * @param string &$name             the parsed resource name
+     * @param string &$type             the parsed resource type
+     * @return void
+     */
+    protected static function parseResourceName($resource_name, $default_resource, &$name, &$type)
+    {
+        $parts = explode(':', $resource_name, 2);
+        if (!isset($parts[1]) || !isset($parts[0][1])) {
+            // no resource given, use default
+            // or single character before the colon is not a resource type, but part of the filepath
+            $type = $default_resource;
+            $name = $resource_name;
+        } else {
+            $type = $parts[0];
+            $name = $parts[1];
+        }
+    }
+
+
+    /**
+     * modify resource_name according to resource handlers specifications
+     *
+     * @param Smarty $smarty        Smarty instance
+     * @param string $resource_name resource_name to make unique
+     * @return string unique resource name
+     */
+
+    /**
+     * modify template_resource according to resource handlers specifications
+     *
+     * @param string $smarty            Smarty instance
+     * @param string $template_resource template_resource to extracate resource handler and name of
+     * @return string unique resource name
+     */
+    public static function getUniqueTemplateName($smarty, $template_resource)
+    {
+        self::parseResourceName($template_resource, $smarty->default_resource_type, $name, $type);
+        // TODO: optimize for Smarty's internal resource types
+        $resource = Smarty_Resource::load($smarty, $type);
+        return $resource->buildUniqueResourceName($smarty, $name);
     }
 
     /**
@@ -393,37 +510,26 @@ abstract class Smarty_Resource {
             $template_resource = $_template->template_resource;
         }
 
+        // parse resource_name, load resource handler, identify unique resource name
+        self::parseResourceName($template_resource, $smarty->default_resource_type, $name, $type);
+        $resource = Smarty_Resource::load($smarty, $type);
+        $unique_resource_name = $resource->buildUniqueResourceName($smarty, $name);
+
         // check runtime cache
-        $_cache_key_dir = join(DIRECTORY_SEPARATOR, $smarty->getTemplateDir());
-        $_cache_key = 'template|' . $template_resource;
-        if (!isset(self::$sources[$_cache_key_dir])) {
-            self::$sources[$_cache_key_dir] = array();
+        $_cache_key = 'template|' . $unique_resource_name;
+        if ($smarty->compile_id) {
+            $_cache_key .= '|'.$smarty->compile_id;
         }
-        if (isset(self::$sources[$_cache_key_dir][$_cache_key])) {
-            return self::$sources[$_cache_key_dir][$_cache_key];
+        if (isset(self::$sources[$_cache_key])) {
+            return self::$sources[$_cache_key];
         }
 
-        if (($pos = strpos($template_resource, ':')) === false) {
-            // no resource given, use default
-            $resource_type = $smarty->default_resource_type;
-            $resource_name = $template_resource;
-        } else {
-            // get type and name from path
-            $resource_type = substr($template_resource, 0, $pos);
-            $resource_name = substr($template_resource, $pos +1);
-            if (strlen($resource_type) == 1) {
-                // 1 char is not resource type, but part of filepath
-                $resource_type = 'file';
-                $resource_name = $template_resource;
-            }
-        }
-
-        $resource = Smarty_Resource::load($smarty, $resource_type);
-        $source = new Smarty_Template_Source($resource, $smarty, $template_resource, $resource_type, $resource_name);
+        // create source
+        $source = new Smarty_Template_Source($resource, $smarty, $template_resource, $type, $name, $unique_resource_name);
         $resource->populate($source, $_template);
 
         // runtime cache
-        self::$sources[$_cache_key_dir][$_cache_key] = $source;
+        self::$sources[$_cache_key] = $source;
         return $source;
     }
 
@@ -435,31 +541,34 @@ abstract class Smarty_Resource {
      */
     public static function config(Smarty_Internal_Config $_config)
     {
+        static $_incompatible_resources = array('eval' => true, 'string' => true, 'extends' => true, 'php' => true);
         $config_resource = $_config->config_resource;
         $smarty = $_config->smarty;
 
-        if (($pos = strpos($config_resource, ':')) === false) {
-            // no resource given, use default
-            $resource_type = $smarty->default_config_type;
-            $resource_name = $config_resource;
-        } else {
-            // get type and name from path
-            $resource_type = substr($config_resource, 0, $pos);
-            $resource_name = substr($config_resource, $pos +1);
-            if (strlen($resource_type) == 1) {
-                // 1 char is not resource type, but part of filepath
-                $resource_type = 'file';
-                $resource_name = $config_resource;
-            }
+        // parse resource_name
+        self::parseResourceName($config_resource, $smarty->default_config_type, $name, $type);
+
+        // make sure configs are not loaded via anything smarty can't handle
+        if (isset($_incompatible_resources[$type])) {
+            throw new SmartyException ("Unable to use resource '{$type}' for config");
         }
 
-        if (in_array($resource_type, array('eval', 'string', 'extends', 'php'))) {
-            throw new SmartyException ("Unable to use resource '{$resource_type}' for config");
+        // load resource handler, identify unique resource name
+        $resource = Smarty_Resource::load($smarty, $type);
+        $unique_resource_name = $resource->buildUniqueResourceName($smarty, $name);
+
+        // check runtime cache
+        $_cache_key = 'config|' . $unique_resource_name;
+        if (isset(self::$sources[$_cache_key])) {
+            return self::$sources[$_cache_key];
         }
 
-        $resource = Smarty_Resource::load($smarty, $resource_type);
-        $source = new Smarty_Config_Source($resource, $smarty, $config_resource, $resource_type, $resource_name);
+        // create source
+        $source = new Smarty_Config_Source($resource, $smarty, $config_resource, $type, $name, $unique_resource_name);
         $resource->populate($source, null);
+
+        // runtime cache
+        self::$sources[$_cache_key] = $source;
         return $source;
     }
 
@@ -523,6 +632,12 @@ class Smarty_Template_Source {
      */
     public $name = null;
 
+    /**
+     * Unique Resource Name
+     * @var string
+     */
+    public $unique_resource = null;
+
     /**
      * Source Filepath
      * @var string
@@ -562,13 +677,14 @@ class Smarty_Template_Source {
     /**
      * create Source Object container
      *
-     * @param Smarty_Resource $handler  Resource Handler this source object communicates with
-     * @param Smarty          $smarty   Smarty instance this source object belongs to
-     * @param string          $resource full template_resource
-     * @param string          $type     type of resource
-     * @param string          $name     resource name
+     * @param Smarty_Resource $handler          Resource Handler this source object communicates with
+     * @param Smarty          $smarty           Smarty instance this source object belongs to
+     * @param string          $resource         full template_resource
+     * @param string          $type             type of resource
+     * @param string          $name             resource name
+     * @param string          $unique_resource  unqiue resource name
      */
-    public function __construct(Smarty_Resource $handler, Smarty $smarty, $resource, $type, $name)
+    public function __construct(Smarty_Resource $handler, Smarty $smarty, $resource, $type, $name, $unique_resource)
     {
         $this->handler = $handler; // Note: prone to circular references
 
@@ -582,6 +698,7 @@ class Smarty_Template_Source {
         $this->resource = $resource;
         $this->type = $type;
         $this->name = $name;
+        $this->unique_resource = $unique_resource;
     }
 
     /**
@@ -593,13 +710,9 @@ class Smarty_Template_Source {
     public function getCompiled(Smarty_Internal_Template $_template)
     {
         // check runtime cache
-        $_cache_key_dir = join(DIRECTORY_SEPARATOR, $_template->smarty->getTemplateDir());
-        $_cache_key = $_template->template_resource . '#' . $_template->compile_id;
-        if (!isset(Smarty_Resource::$compileds[$_cache_key_dir])) {
-            Smarty_Resource::$compileds[$_cache_key_dir] = array();
-        }
-        if (isset(Smarty_Resource::$compileds[$_cache_key_dir][$_cache_key])) {
-            return Smarty_Resource::$compileds[$_cache_key_dir][$_cache_key];
+        $_cache_key = $this->unique_resource . '#' . $_template->compile_id;
+        if (isset(Smarty_Resource::$compileds[$_cache_key])) {
+            return Smarty_Resource::$compileds[$_cache_key];
         }
 
         $compiled = new Smarty_Template_Compiled($this);
@@ -608,7 +721,7 @@ class Smarty_Template_Source {
         $compiled->exists = !!$compiled->timestamp;
 
         // runtime cache
-        Smarty_Resource::$compileds[$_cache_key_dir][$_cache_key] = $compiled;
+        Smarty_Resource::$compileds[$_cache_key] = $compiled;
 
         return $compiled;
     }
